%% output OpenFDTD data file
function output_ofd(fn, Parm, Epsr, Sigm)
%%
[fid, errmsg] = fopen(fn, 'wt');
if (fid < 0)
	fprintf('%s : %s\n', errmsg, fn);
	return;
end

mfmt = 'material = 1 %g %g 1 0\n';
gfmt = 'geometry = %d 1 %g %g %g %g %g %g\n';
ffmt = 'feed = %s %g %g %g %g %g %g\n';
lfmt = 'load = %s %g %g %g %s %g\n';
pfmt = 'point = %s %g %g %g\n';
dir = ['X' 'Y' 'Z'];
rcl = ['R' 'C' 'L'];

nv = 1;  % =1/2...
d = 1e-3;  % fix
margin = 20;
nxv = floor(Parm.nxp / nv);
nyv = floor(Parm.nyp / nv);
nzv = floor(Parm.nzp / nv);
sx = margin * d;
sy = margin * d;
sz = margin * d;
x0 = - (Parm.nxp / 2 * d) - sx;
y0 = - (Parm.nyp / 2 * d) - sy;
z0 =                - sz;
x1 = x0 + (nxv * nv * d) + (2 * sx);
y1 = y0 + (nyv * nv * d) + (2 * sy);
z1 = z0 + (nzv * nv * d) + (2 * sz);

% title
fprintf(fid, 'OpenFDTD 3 1\n');
fprintf(fid, 'title = matlab\n');

% mesh
fprintf(fid, 'xmesh = %g %d %g\n', x0, round((x1 - x0) / (nv * d)), x1);
fprintf(fid, 'ymesh = %g %d %g\n', y0, round((y1 - y0) / (nv * d)), y1);
fprintf(fid, 'zmesh = %g %d %g\n', z0, round((z1 - z0) / (nv * d)), z1);

% background (material=2)
fprintf(fid, mfmt, Parm.epsr(1), Parm.sigm(1));
fprintf(fid, gfmt, 2, x0, x1, y0, y1, z0, z1);

% phantom
m = 2;

% X-line
for k = nv : nv : Parm.nzp - nv
for j = nv : nv : Parm.nyp - nv
for i = nv : nv : Parm.nxp
	epsr = (Epsr(i, j + 1, k + 1) ...
	      + Epsr(i, j,     k + 1) ...
	      + Epsr(i, j + 1, k    ) ...
	      + Epsr(i, j,     k    )) / 4;
	sigm = (Sigm(i, j + 1, k + 1) ...
	      + Sigm(i, j,     k + 1) ...
	      + Sigm(i, j + 1, k    ) ...
	      + Sigm(i, j,     k    )) / 4;
	fprintf(fid, mfmt, epsr, sigm);
	gx1 = (i - nv - Parm.nxp / 2) * d;
	gx2 = (i -  0 - Parm.nxp / 2) * d;
	gy = (j - Parm.nyp / 2) * d;
	gz = k * d;
	m = m + 1;
	fprintf(fid, gfmt, m, gx1, gx2, gy, gy, gz, gz);
end
end
end

% Y-line
for k = nv : nv : Parm.nzp - nv
for j = nv : nv : Parm.nyp
for i = nv : nv : Parm.nxp - nv
	epsr = (Epsr(i + 1, j, k + 1) ...
	      + Epsr(i + 1, j, k    ) ...
	      + Epsr(i,     j, k + 1) ...
	      + Epsr(i,     j, k    )) / 4;
	sigm = (Sigm(i + 1, j, k + 1) ...
	      + Sigm(i + 1, j, k    ) ...
	      + Sigm(i,     j, k + 1) ...
	      + Sigm(i,     j, k    )) / 4;
	fprintf(fid, mfmt, epsr, sigm);
	gx = (i - Parm.nxp / 2) * d;
	gy1 = (j - nv - Parm.nyp / 2) * d;
	gy2 = (j -  0 - Parm.nyp / 2) * d;
	gz = k * d;
	m = m + 1;
	fprintf(fid, gfmt, m, gx, gx, gy1, gy2, gz, gz);
end
end
end

% Z-line
for k = nv : nv : Parm.nzp
for j = nv : nv : Parm.nyp - nv
for i = nv : nv : Parm.nxp - nv
	epsr = (Epsr(i + 1, j + 1, k) ...
	      + Epsr(i,     j + 1, k) ...
	      + Epsr(i + 1, j,     k) ...
	      + Epsr(i,     j,     k)) / 4;
	sigm = (Sigm(i + 1, j + 1, k) ...
	      + Sigm(i,     j + 1, k) ...
	      + Sigm(i + 1, j,     k) ...
	      + Sigm(i,     j,     k)) / 4;
	fprintf(fid, mfmt, epsr, sigm);
	gx = (i - Parm.nxp / 2) * d;
	gy = (j - Parm.nyp / 2) * d;
	gz1 = (k - nv) * d;
	gz2 = (k -  0) * d;
	m = m + 1;
	fprintf(fid, gfmt, m, gx, gx, gy, gy, gz1, gz2);
end
end
end

% antenna : feed + load + point
Ant = setup_antenna();
itx = 1;
for na = 1 : size(Ant, 1)
	da = dir(Ant(na, 1));
	xa = Ant(na, 2);
	ya = Ant(na, 3);
	za = Ant(na, 4);
	ha = Ant(na, 5) / 2;
	if     (da == 'X')
		fprintf(fid, gfmt, 1, xa - ha, xa + ha, ya, ya, za, za);
	elseif (da == 'Y')
		fprintf(fid, gfmt, 1, xa, xa, ya - ha, ya + ha, za, za);
	elseif (da == 'Z')
		fprintf(fid, gfmt, 1, xa, xa, ya, ya, za - ha, za + ha);
	end
	if (na == itx)
		% feed
		amp = 1;
		phs = 0;
		z0 = 50;
		fprintf(fid, ffmt, da, xa, ya, za, amp, phs, z0);
	else
		% load
		r = 50;
		fprintf(fid, lfmt, da, xa, ya, za, rcl(1), r);
		% point
		fprintf(fid, pfmt, da, xa, ya, za);
	end
end

% misc
fprintf(fid, 'frequency1 = %g %g 0\n', Parm.freq, Parm.freq);
fprintf(fid, 'frequency2 = %g %g 0\n', Parm.freq, Parm.freq);
fprintf(fid, 'abc = 1 8 3 1e-8\n');  % PML
fprintf(fid, 'solver = 10000 200 1e-3\n');

% post
%fprintf(fid, 'plotiter = 1\n');
fprintf(fid, 'plotnear2d = E Z %g\n', 10 * d);
%fprintf(fid, 'plotnear2d = E Z %g\n', 18 * d);
%fprintf(fid, 'plotnear2d = E X %g\n', 0 * d);
fprintf(fid, 'near2ddim = 1 0\n');
%fprintf(fid, 'near2ddb = 1\n');
%fprintf(fid, 'near2dscale = -15 30 10\n');
fprintf(fid, 'near2dcontour = 1\n');
fprintf(fid, 'near2dobj = 0\n');
fprintf(fid, 'near2dzoom = -50e-3 50e-3 -50e-3 50e-3\n');
%fprintf(fid, 'near2dzoom = -50e-3 50e-3 0e-3 50e-3\n');
%fprintf(fid, 'window2d = 600 400 12 0\n');
%fprintf(fid, 'window3d = 600 600 12 0 60 30\n');
fprintf(fid, 'end\n');

fclose(fid);
end

%% antenna
function Ant = setup_antenna()
%%
Ant = zeros(4, 5);
ra = 60e-3;
za = 25e-3;
la = 26e-3;

% +X
n = 1;
Ant(n, 1) = 3;  % 1/2/3=X/Y/Z
Ant(n, 2) = +ra;
Ant(n, 3) = 0;
Ant(n, 4) = za;
Ant(n, 5) = la;

% -X
n = 2;
Ant(n, 1) = 3;
Ant(n, 2) = -ra;
Ant(n, 3) = 0;
Ant(n, 4) = za;
Ant(n, 5) = la;

% +Y
n = 3;
Ant(n, 1) = 3;
Ant(n, 2) = 0;
Ant(n, 3) = +ra;
Ant(n, 4) = za;
Ant(n, 5) = la;

% -Y
n = 4;
Ant(n, 1) = 3;
Ant(n, 2) = 0;
Ant(n, 3) = -ra;
Ant(n, 4) = za;
Ant(n, 5) = la;
end
